package board;

import objects.GameObject;
import objects.Player;

import java.util.List;

public interface IBoard extends IGrid<GameObject> {
    /**
     * Adds a new player to the board.
     *
     * @param p The player to add
     * @return  True if the player is valid
     */
    boolean addPlayer(Player p);

    /**
     * Gets the next player from the board.
     *
     * @return  A player
     */
    Player nextPlayer();

    /**
     * Find out if someone has won.
     *
     * @return True if someone won
     */
    boolean won();

    /**
     * Flushes all values in the board.
     */
    void flush();

    /**
     * Gets the position of the topmost object in the board's grid.
     *
     * @param column    The target column
     * @return          A position or -1 if all rows are empty
     */
    int getTopObject(int column);

    /**
     * Gets a copy of the players list.
     *
     * @return  A list of players
     */
    List<Player> getPlayers();
}
