package interfaces;

import board.Board;
import events.IWinCondition;
import events.ThreeInARow;
import objects.Empty;
import objects.GameObject;
import objects.Player;
import objects.Token;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;

public class TicTacToeGUI extends GUI {
    private JButton[] gameGrid = new JButton[9];

    public TicTacToeGUI() {
        List<IWinCondition> conditions = new ArrayList<>();
        conditions.add(new ThreeInARow());
        this.board = new Board(3,3, new Empty(), conditions);
        int playerNum = 2;
        for (int i = 0; i < playerNum; i++) {
            String playerName = "";
            while (playerName.equals("")) {
                playerName = JOptionPane.showInputDialog(String.format("Name of player %d: ", i+1));
                if (playerName == null) {
                    System.exit(0);
                }
                if (playerName.length() > 10) {
                    JOptionPane.showMessageDialog(null, "Player name can't be longer than 10 characters.", "Error", JOptionPane.ERROR_MESSAGE);
                    playerName = "";
                }
            }
            Player player;
            if (i == 1) {
                player = new Player(playerName, Color.BLACK);
            } else {
                player = new Player(playerName, Color.WHITE);
            }
            if (!this.board.addPlayer(player)) {
                JOptionPane.showMessageDialog(null, "Player is not unique. Please try again.", "Error", JOptionPane.ERROR_MESSAGE);
                i--;
            }
        }
        initialize();
    }

    private void initialize() {
        JFrame frame = new JFrame("Four in a row");
        frame.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        int fullWidth = (BUTTON_WIDTH) * 3;
        int fullHeight = (TILE_HEIGHT) * 3;

        this.frame = frame;
        JPanel container = new JPanel();
        container.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));
        container.setPreferredSize(new Dimension(fullWidth, fullHeight + 2 * BUTTON_HEIGHT));
        frame.add(container);

        JPanel panel2 = new JPanel();
        this.mainPanel = panel2;
        panel2.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));
        panel2.setPreferredSize(new Dimension(fullWidth, fullHeight));
        readTiles();
        container.add(panel2);

        infoLabel = new JLabel();
        infoLabel.setPreferredSize(new Dimension(fullWidth, 15));
        infoLabel.setBorder(new EmptyBorder(10, 10, 15, 10));
        container.add(infoLabel);
        this.resetButton = new JButton("Reset");
        this.resetButton.addActionListener(this);
        container.add(this.resetButton);

        frame.validate();
        frame.pack();
        frame.setVisible(true);
        currentPlayer = board.nextPlayer();
        displayMessage("It's " + currentPlayer + "'s turn.");
    }

    public void displayBoard() {
        for (JButton button : this.gameGrid) {
            this.mainPanel.remove(button);
        }
        this.gameGrid = new JButton[this.board.getWidth() * this.board.getHeight()];
        readTiles();
        this.frame.validate();
    }

    void readTiles() {
        int k = 0;
        for (int i = 0; i < board.getHeight(); i++) {
            for (int j = 0; j < board.getWidth(); j++) {
                JButton button = new JButton();
                button.addActionListener(this);
                GameObject obj = board.get(j,i);
                if (obj.getColor() == Color.black) {
                    button.setText("X");
                } else if (obj.getColor() == Color.white) {
                    button.setText("O");
                }
                button.setPreferredSize(new Dimension(BUTTON_WIDTH, TILE_HEIGHT));
                this.gameGrid[k++] = button;
                mainPanel.add(button, BorderLayout.PAGE_START);
            }
        }
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == this.resetButton) {
            this.board.flush();
            displayBoard();
            this.currentPlayer = this.board.nextPlayer();
            displayMessage("It's " + this.currentPlayer + "'s turn.");
        }
        if (!(board.won() || board.isFull())) {
            for (int i = 0; i < this.gameGrid.length; i++) {
                if (e.getSource() == this.gameGrid[i]) {
                    if (this.gameGrid[i].getText().equals("")) {
                        int x = i % 3;
                        int y = i / 3;
                        this.board.set(x, y, new Token(currentPlayer));
                        displayBoard();
                        if (!board.won()) {
                            currentPlayer = board.nextPlayer();
                            displayMessage("It's " + currentPlayer + "'s turn.");
                        } else {
                            GameObject winnerObject = board.get(x, board.getTopObject(x));
                            displayMessage(((Token)winnerObject).getOwner() + " won.");
                        }
                        if (board.isFull()) {
                            displayMessage("It's a draw");
                        }
                    }
                }
            }
        }
    }
}
